@@ -67,7 +67,6 @@ typedef enum : unsigned {
         VTE_DEBUG_BIDI          = 1u << 28,
         VTE_DEBUG_CONVERSION    = 1u << 29,
         VTE_DEBUG_EXCEPTIONS    = 1u << 30,
-        VTE_DEBUG_IMAGE         = 1u << 31,
 } VteDebugFlags;
 
 void _vte_debug_init(void);
@@ -87,13 +86,13 @@ _vte_debug_on(guint flags)
 	return (_vte_debug_flags & flags) != 0;
 }
 
-#ifdef VTE_DEBUG
+#if VTE_DEBUG
 #define _VTE_DEBUG_IF(flags) if (G_UNLIKELY (_vte_debug_on (flags)))
 #else
 #define _VTE_DEBUG_IF(flags) if (0)
 #endif
 
-#ifdef VTE_DEBUG
+#if VTE_DEBUG
 #if defined(__GNUC__) && G_HAVE_GNUC_VARARGS
 #define _vte_debug_print(flags, fmt, ...) \
 	G_STMT_START { _VTE_DEBUG_IF(flags) g_printerr(fmt, ##__VA_ARGS__); } G_STMT_END
