@@ -15,14 +15,6 @@
  * along with this library.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-/*
- * SECTION: vte-access
- * @short_description: Accessibility peer of #VteTerminal
- *
- * The accessibility peer of a #VteTerminal, implementing GNOME's accessibility
- * framework.
- */
-
 #include "config.h"
 
 #include <atk/atk.h>
@@ -35,7 +27,7 @@
 #include "vteinternal.hh"
 #include "widget.hh"
 
-#ifdef HAVE_LOCALE_H
+#if __has_include(<locale.h>)
 #include <locale.h>
 #endif
 #include <glib/gi18n-lib.h>
@@ -378,11 +370,21 @@ vte_terminal_accessible_maybe_emit_text_caret_moved(VteTerminalAccessible *acces
         }
 }
 
-/* A signal handler to catch "text-inserted/deleted/modified" signals. */
-static void
-vte_terminal_accessible_text_modified(VteTerminal *terminal, gpointer data)
+void
+_vte_terminal_accessible_text_inserted(VteTerminalAccessible* accessible)
+{
+        _vte_terminal_accessible_text_modified(accessible);
+}
+
+void
+_vte_terminal_accessible_text_deleted (VteTerminalAccessible* accessible)
+{
+        _vte_terminal_accessible_text_modified(accessible);
+}
+
+void
+_vte_terminal_accessible_text_modified(VteTerminalAccessible* accessible)
 {
-        VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
         GString *old_text;
         GArray *old_characters;
@@ -491,13 +493,10 @@ vte_terminal_accessible_text_modified(VteTerminal *terminal, gpointer data)
         g_array_free(old_characters, TRUE);
 }
 
-/* A signal handler to catch "text-scrolled" signals. */
-static void
-vte_terminal_accessible_text_scrolled(VteTerminal *terminal,
-				      gint howmuch,
-				      gpointer data)
+void
+_vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
+                                       long howmuch)
 {
-        VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 	struct _VteCharAttributes attr;
 	long delta, row_count;
@@ -507,6 +506,9 @@ vte_terminal_accessible_text_scrolled(VteTerminal *terminal,
         /* g_assert(howmuch != 0); */
         if (howmuch == 0) return;
 
+        auto widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
+        auto terminal = VTE_TERMINAL(widget);
+
         row_count = vte_terminal_get_row_count(terminal);
 	if (((howmuch < 0) && (howmuch <= -row_count)) ||
 	    ((howmuch > 0) && (howmuch >= row_count))) {
@@ -625,7 +627,8 @@ vte_terminal_accessible_text_scrolled(VteTerminal *terminal,
 			if (len > i) {
 				/* snapshot_text always contains a trailing '\n',
 				 * insertion happens in front of it: bug 657960 */
-				g_assert(i >= 1);
+				// g_assert(i >= 1);
+                            if (i > 0)
 				emit_text_changed_insert(G_OBJECT(accessible),
 							 priv->snapshot_text->str,
 							 i - 1,
@@ -707,31 +710,21 @@ vte_terminal_accessible_selection_changed (VteTerminal *terminal,
 }
 
 static void
-vte_terminal_accessible_initialize (AtkObject *obj, gpointer data)
+vte_terminal_accessible_initialize(AtkObject *obj,
+                                   gpointer data)
 {
-	VteTerminal *terminal = VTE_TERMINAL (data);
+        auto accessible = VTE_TERMINAL_ACCESSIBLE(obj);
+        VteTerminal *terminal = VTE_TERMINAL(data);
         const char *window_title;
 
 	ATK_OBJECT_CLASS (_vte_terminal_accessible_parent_class)->initialize (obj, data);
 
         auto impl = IMPL(terminal);
         try {
-                impl->subscribe_accessible_events();
+                impl->set_accessible(accessible);
         } catch (...) {
         }
 
-	g_signal_connect(terminal, "text-inserted",
-			 G_CALLBACK(vte_terminal_accessible_text_modified),
-			 obj);
-	g_signal_connect(terminal, "text-deleted",
-			 G_CALLBACK(vte_terminal_accessible_text_modified),
-			 obj);
-	g_signal_connect(terminal, "text-modified",
-			 G_CALLBACK(vte_terminal_accessible_text_modified),
-			 obj);
-	g_signal_connect(terminal, "text-scrolled",
-			 G_CALLBACK(vte_terminal_accessible_text_scrolled),
-			 obj);
 	g_signal_connect(terminal, "cursor-moved",
 			 G_CALLBACK(vte_terminal_accessible_invalidate_cursor),
 			 obj);
@@ -787,16 +780,6 @@ vte_terminal_accessible_finalize(GObject *object)
         widget = gtk_accessible_get_widget (GTK_ACCESSIBLE(accessible));
 
 	if (widget != NULL) {
-		g_signal_handlers_disconnect_matched(widget,
-						     (GSignalMatchType)(G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA),
-						     0, 0, NULL,
-						     (void *)vte_terminal_accessible_text_modified,
-						     object);
-		g_signal_handlers_disconnect_matched(widget,
-						     (GSignalMatchType)(G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA),
-						     0, 0, NULL,
-						     (void *)vte_terminal_accessible_text_scrolled,
-						     object);
 		g_signal_handlers_disconnect_matched(widget,
 						     (GSignalMatchType)(G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA),
 						     0, 0, NULL,
@@ -812,6 +795,13 @@ vte_terminal_accessible_finalize(GObject *object)
 						     0, 0, NULL,
 						     (void *)vte_terminal_accessible_visibility_notify,
 						     object);
+
+                auto terminal = VTE_TERMINAL(widget);
+                auto impl = IMPL(terminal);
+                try {
+                        impl->set_accessible(nullptr);
+                } catch (...) {
+                }
 	}
 
 	if (priv->snapshot_text != NULL) {
@@ -1592,8 +1582,8 @@ vte_terminal_accessible_set_size(AtkComponent *component,
         /* If the size is an exact multiple of the cell size, use that,
          * otherwise round down. */
         try {
-                width -= impl->m_padding.left + impl->m_padding.right;
-                height -= impl->m_padding.top + impl->m_padding.bottom;
+                width -= impl->m_border.left + impl->m_border.right;
+                height -= impl->m_border.top + impl->m_border.bottom;
 
                 auto columns = width / impl->m_cell_width;
                 auto rows = height / impl->m_cell_height;
